<?php
/**
 * 
 * @copyright Copyright(c) 2005 Daniele Ugoletti. All rights reserved.
 * @license http://www.gnu.org/copyleft/lesser.html GNU LESSER GENERAL PUBLIC LICENSE
 *
 * GLIZY is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 * See COPYRIGHT.php for copyright notices and details.
 *
 * @author		Daniele Ugoletti <daniele@ugoletti.com>
 * @version		
 * @package		Glizy core
 * @category	
 */

core_import('org.glizy.dataAccess.*');

class org_glizy_dataAccessHasMany extends org_glizy_dataAccess_relation
{	
	var $recordSet		= NULL;
	var $iterator		= NULL;
	var $_recordsList	= array();

	function  org_glizy_dataAccessHasMany(&$parent, $options)
	{
		parent::org_glizy_dataAccess_relation($parent, $options);
		assert(isset($options['field']));
		$this->_key = $options['field'];
		assert(!is_null($this->_destinationKey));
		$this->_destinationKey = $options['destinationField'];
	}
	
	function build($params=array())
	{
		$this->record 	= &org_glizy_dataAccess_DataAccess::getObject($this->_className);
		$this->iterator	= &$this->record->loadQuery('All', array('filters' => array($this->_destinationKey => $this->_parent->{$this->_key})));
		
		if ($this->iterator->count())
		{
			$this->_current();
		}
		else
		{
			$this->record = NULL;
		}
	}
	
	function create($params)
	{
		$this->_reset();
		$this->record = & org_glizy_dataAccess_DataAccess::getObject($this->_className);
		$this->record->setProcessRelations(false);

		if (count($params))
		{
			$this->record->loadFromArray($params);
		}
		$this->_bindRecordFields();
	}

	function bind(&$object)
	{
		if (get_class($object)!=$this->_className)
		{
			// TODO
			// visualizzare errore
		}
		
		$this->_reset();
		$this->record = &$object;
		$this->record->{$this->_destinationKey} 		= $this->_parent->{$this->_key};
		$this->_recordsList[$this->record->getId()] 	= &$this->record;
		$this->_backUpValues[$this->record->getId()] 	= $this->record->getValuesAsArray(); 
		$this->record->setProcessRelations(false);
		$this->_bindRecordFields();
	}

	function postSave()
	{
		if (is_null($this->record->{$this->_destinationKey}))
		{
			// nuovo record
			$this->record->{$this->_destinationKey} = $this->_parent->{$this->_key};
			$newId = $this->record->save();
			$this->_backUpValues[$newId] = $this->record->getValuesAsArray();
		}
		else
		{
			foreach($this->_recordsList as $id=>$obj)
			{
				$canSave = false;
				$currentValues = $this->_recordsList[$id]->getValuesAsArray();
				foreach ($currentValues as $k=>$v)
				{
					if (!array_key_exists($k, $this->_backUpValues[$this->_recordsList[$id]->getId()]) || $this->_backUpValues[$this->_recordsList[$id]->getId()][$k]!=$v)
					{
						$canSave = true;
						break;
					}
				}
				
				if ($canSave) 
				{
					$this->_recordsList[$id]->save();
					$this->_backUpValues[$this->_recordsList[$id]->getId()] = $this->_recordsList[$id]->getValuesAsArray();
				}
			}
		}
	}

	function &getIterator()
	{
		return $this->iterator;
	}
	
	/* implementa gli stessi metodi dell'iterator */
	function rewind() 
	{
		assert(!is_null($this->iterator));
        $this->iterator->rewind();
		$this->_current();
   }

	function valid() 
	{
		assert(!is_null($this->iterator));
        return $this->iterator->valid();
    }
	
	function &first() 
	{
		assert(!is_null($this->iterator));
        $this->iterator->first();
   		$this->_current();
		return $this->record;
	}

    function key() 
	{
		assert(!is_null($this->iterator));
        return $this->iterator->key();
    }
		
    function next() 
	{
		assert(!is_null($this->iterator));

		$this->iterator->next();
		if ($this->iterator->hasMore())
		{
			$this->_current();
		}
    }
	
	function count()
	{
		assert(!is_null($this->iterator));
		return $this->iterator->count();
	}
	
	function hasMore()
	{
		assert(!is_null($this->iterator));
		return $this->iterator->hasMore();
	}

	function _current()
	{
		assert(!is_null($this->iterator));
		$this->record = &$this->iterator->current();
		$this->_recordsList[$this->record->getId()] = &$this->record;
		$this->_backUpValues[$this->record->getId()] = $this->record->getValuesAsArray(); 
		$this->_bindRecordFields();
	}

	function _reset()
	{
		$this->record			= NULL;
		$this->iterator			= NULL;
		$this->_recordsList		= array();	
		$this->_backUpValues	= array();	
	}
}
?>